<?php

namespace App\Http\Controllers\Admin\Structure;

use App\Filter\FilterClass\Title;
use App\Http\Controllers\Controller;
use App\Http\Requests\Admin\MangeAdmin\UpdateAdminRequest;
use App\Http\Requests\Admin\StoreIndustryRequest;
use App\Http\Requests\Admin\UpdateIndustryRequest;
use App\Models\JobTitle;
use App\Traits\ApiResponser;
use App\Traits\Helper;
use App\Traits\pipeline;
use Illuminate\Http\Request;

class IndustryController extends Controller
{
    use pipeline, Helper, ApiResponser;

    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        return $this->successResponseWithoutMessage(true, $this->pipeline(JobTitle::query(), [Title::class]));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        //
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @return \Illuminate\Http\Response
     */
    public function store(StoreIndustryRequest $request)
    {
        $validated = $request->validated();
        $validated['photo'] = $this->photo($request->file('photo'));
        $industry = JobTitle::create($validated);
        return ($industry) ? $this->successResponseWithoutMessage(true, $industry) :
            $this->failedResponse('The industry in question was not registered');
    }

    /**
     * Display the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {

    }

    /**
     * Update the specified resource in storage.
     *
     * @param \Illuminate\Http\Request $request
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function update(UpdateIndustryRequest $request, $id)
    {
        $industry = JobTitle::find($id);

        if ($industry) {
            $validated = $request->validated();
            $validated['photo'] = $this->photo($request->file('photo'));
            $adminIndustry=$industry->update($validated);
          return  ($adminIndustry)?$this->successResponseWithoutMessage(true,JobTitle::find($id) ):$this->failedResponse('The industry in question was not registered');
        }else {
            $this->failedResponse('industry not found');
        }
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param int $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $industry = JobTitle::find($id);
        return ($industry) ? $this->successResponseWithoutMessage(true, $industry->delete()) :
            $this->failedResponse('industry not find');
    }
}
