<?php

namespace App\Http\Controllers\Client;

//use App\Filter\FilterClass\User\Q;
//use App\Filter\FilterClass\Company\Q;
use App\Helper\ApiResponserHelper;
use App\Http\Controllers\Controller;
use App\Http\Controllers\ImageController;
use App\Http\Controllers\SocialMediaController;
use App\Http\Requests\ShowInformationUserForOthersRequest;
use App\Http\Resources\ListClientsResource;
use App\Http\Resources\PaginationResource;
use App\Http\Resources\Profile\ShowOneProjectGropeResuorce;
use App\Http\Resources\ShowProfileResource;
use App\Models\Company;
use App\Models\GeneralContractor;
use App\Models\Owner;
use App\Models\ParticipantsProject;
use App\Models\Project;
use App\Models\SocialMedia;
use App\Models\SocialMediable;
use App\Models\User;
use App\Repositories\JobTitleRepository;
use App\Repositories\Role\SettingInformationRepository;
use App\Services\AddressService;
use App\Services\OwnerService;
use App\Traits\ApiResponser;
use App\Traits\Helper;
use App\Traits\pipeline;
use Illuminate\Http\Request;
use Illuminate\Support\Arr;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\Str;
use Illuminate\Validation\Rule;
use Mojtaba\Chatable\Models\Group;
use Mojtaba\Chatable\Models\Member;
use Symfony\Component\HttpFoundation\Response;

class ProfileController extends Controller
{
    use ApiResponser,Helper,pipeline;

    public $request;
    public $clientType;
    public $clientId;
    public $clientEntity;

    public function __construct(Request $request)
    {
        $this->request = $request;
    }

    public function call($clientRole)
    {
        $route = Route::current()->getName();
        $methodName = "$route" . Str::ucfirst($clientRole);
        $this->clientEntity = auth($clientRole)->user();
        $this->clientId = $this->clientEntity->id;
        $this->clientType = 'App\\Models\\' . Str::ucfirst($clientRole);
        return $this->{$methodName}();
    }

    public function completeInformationUser()
    {
        $validatedData = $this->request->validate([
            'first_name' => ['required', 'min:3', 'max:100', 'regex:/^[a-zA-Z\s]+$/'],
            'last_name' => ['required', 'min:3', 'max:100', 'regex:/^[a-zA-Z\s]+$/'],
            'phone_number' => ['required', 'max:20', 'min:3', 'string']
        ]);
        $repository = new SettingInformationRepository();
        $data = $repository->completeInformationUser($validatedData, $this->clientId);
        return successResponse(true, $data, 'your information save successfully');
    }

    public function completeInformationCompany()
    {
        //validate
        $validatedData = $this->request->validate([
            'name' => ['required', 'min:3', 'max:100', 'regex:/^[a-zA-Z\s]+$/',
                Rule::unique('companies', 'name')->ignore($this->clientId)
            ],
            'company_phone_number' => ['required', 'min:3', 'max:50'],
            'job_title' => [ 'max:30', 'min:3'],
            'main_text' => ['required', 'min:3', 'max:50', 'max:250'],
            'secondary_text' => ['min:3', 'max:50', 'max:250'],
            'country' => ['required', 'min:2', 'max:50', 'regex:/^[a-zA-Z\s]+$/'],
            'city' => ['required', 'min:2', 'max:50', 'regex:/^[a-zA-Z\s]+$/'],
            'state' => ['required', 'min:2', 'max:50', 'regex:/^[a-zA-Z\s]+$/'],
            'zip_code' => ['required', 'digits_between:1,20', 'numeric']
        ]);

        //dd($validatedData);
        //add job
        $jobId = null;
        if (array_key_exists('job_title', $validatedData)) {
            $jobRepository = new JobTitleRepository();
            $job = $jobRepository->findJob($this->request['job_title']);
            if ($job == null) {
                return failedResponse('Not Found job', Response::HTTP_NOT_FOUND);
            }
            $jobId = $job->toArray()['id'];
        }


        $repository = new SettingInformationRepository();
        $data = $repository->completeInformationCompany($this->request, $this->clientId, $jobId);

        //add address
        $addressService = new AddressService();
        $checkAddress = $addressService->checkExist($this->clientType, $this->clientId);
        if ($checkAddress) {
            $updateAddress = $addressService->update($validatedData, $checkAddress->id);
        } else {
            $address = $addressService->store($this->request, $this->clientType, $this->clientId);
        }
        return successResponse(true, null, 'your information save successfully');
    }

    public function showInformationUser()
    {
//        return 1;
    $type=$this->clientType;
    $id=$this->clientId;
//        $list = $this->clientEntity->load(['socialMedias', 'jobTitle']);
        $return=  $this->clientEntity::query()->withAvg('profileComments','rate')->withCount('participantsProject')->find( $this->clientId)->load(['jobTitle']) ;
        $social= SocialMediable::query()->with('SocialMedia')->where('social_mediable_type',$this->clientType)->where('social_mediable_id',$this->clientId)->get();

        $return['social']=ShowProfileResource::collection($social);
        $participants=    ParticipantsProject::query()->where('participants_project_type' ,$this->clientType)->where('participants_project_id',$this->clientId)
            ->pluck('project_id');
        $participantsCompany=ParticipantsProject::query()->whereIn('project_id',$participants)->where('participants_project_type' ,'App\Models\Company')->pluck('participants_project_id');
//        return $participantsCompany;
        $return['company_image']= Company::query()->whereIn('id',$participantsCompany)->select('id','image')->get();
        $grope=  Group::query()->with('members.client')
            ->whereHas('members',function ($q) use ($id,$type){
            $q->where('member_id',$id)->where('member_type',$type);
        }
        )->get();

        $return['group_image']=ShowOneProjectGropeResuorce::collection($grope);
        return successResponse(true, $return, 'send successfully');

//return successResponse(true, $list, 'send successfully');
    }
    public function showInformationCompany()
    {
//        return 1;
//        $list = $this->clientEntity->load(['socialMedias', 'jobTitle']);
        $return['company']= Company::query()->where('id', auth('company')->user()->id)->get()->load(['jobTitle','address','owner']) ;
        $social= SocialMediable::query()->with('SocialMedia')->where('social_mediable_type',$this->clientType)->where('social_mediable_id',$this->clientId)->get();
        $return['social']=ShowProfileResource::collection($social);
        return successResponse(true, $return, 'send successfully');

//return successResponse(true, $list, 'send successfully');
    }

//    public function showInformationCompany()
//    {
//        $return=  $this->clientEntity::find( $this->clientId)->load(['jobTitle','address','owner']) ;
//        $social= SocialMediable::query()->with('SocialMedia')->where('social_mediable_type',$this->clientType)->where('social_mediable_id',$this->clientId)->get();
//
//        $return['social']=ShowProfileResource::collection($social);
//        return successResponse(true, $return, 'send successfully');
//    }

    public function updateInformationUser()
    {
//        return 22;
 //        return $this->request->image;
//        Log::info($this->request);
        //validate data
        $validatedData = $this->request->validate([
            'first_name' => ['required', 'regex:/^[a-zA-Z\s]+$/', 'max:100', 'min:3'],
            'last_name' => ['required', 'regex:/^[a-zA-Z\s]+$/', 'max:100', 'min:3'],
            'image' => 'image|mimes:jpg,png,jpeg,gif,svg|max:2048',
            'phone_number' => ['max:20', 'min:3'],
            'work_experience' => [ 'max:11','min:2'],
            'job_title' => [ 'exists:job_titles,title','max:30', 'min:3'],
            'about_us' => 'max:600',
            'social_media.*' => ['array'],
            'social_media.*.review' => ['nullable','between:1,10'],
            'social_media.*.link' => ['required_with:social_media.*.review', 'min:1']
        ]);
        //add social media
        if (isset($this->request['social_media'])) {
//            return 2;
            $media = new SocialMediaController();
            $media->storeSocialMedia($this->clientId, $this->clientType, $this->request['social_media']);
        }
        //add image
        $imageName = null;
        if ($this->request['image']) {
            $imageController = new ImageController();
            $imageName = $imageController->upload($this->request->image);
        }
//return $imageName ;
        //add job
        $jobId = null;
        if (isset($this->request['job_title'])) {
            $jobRepository = new JobTitleRepository();
            $job = $jobRepository->findJob($this->request['job_title']);
            if ($job == null) {
                return failedResponse('Not Found job', Response::HTTP_NOT_FOUND);
            }
            $jobId = $job->toArray()['id'];
        }
        $settingInformationRepository = new SettingInformationRepository();

        $user = $settingInformationRepository->updateInformationUser($this->request, $this->clientId, $jobId, $imageName);
        $return=  $this->clientEntity::find( $this->clientId)->load(['jobTitle','address']) ;
        $social= SocialMediable::query()->with('SocialMedia')->where('social_mediable_type',$this->clientType)->where('social_mediable_id',$this->clientId)->get();
        $return['social']=ShowProfileResource::collection($social);
        return successResponse(true, $return, 'send successfully');    }
//        return successResponse(true, $this->clientEntity::find( $this->clientId)->load(['socialMedias', 'jobTitle']) , 'send successfully');    }


    public function updateInformationCompany()
    {
//                return 22;

        //validate
        $validateData = $this->request->validate([
            'name' => ['required', 'max:255', 'min:3',
                Rule::unique('companies', 'name')->ignore($this->clientId)],
            'image' => ['image', 'mimes:jpg,png,jpeg,gif,svg', 'max:2048'],
            'company_phone_number' => ['required','max:20', 'min:3'],
            'about_us' => ['max:600'],
            'country' => [ 'min:2', 'max:50', 'regex:/^[a-zA-Z\s]+$/'],
            'city' => [ 'min:2', 'max:50', 'regex:/^[a-zA-Z\s]+$/'],
            'state' => [ 'min:2', 'max:50', 'regex:/^[a-zA-Z\s]+$/'],
            'job_title' => ['required', 'max:30'],
            'zip_code' => [ 'numeric', 'digits_between:1,20'],
            'main_text' => ['min:2', 'max:200'],
            'secondary_text' => ['min:2', 'max:200'],
            'social_media.*.*' => [ 'max:200'],
            'owner.first_name' => ['size:1'],
            'owner.last_name' => [ 'regex:/^[a-zA-Z\s]+$/', 'min:2', 'max:40'],
            'owner.company_name' => [ 'regex:/^[a-zA-Z\s]+$/', 'min:2', 'max:40'],
            'owner.email' => [ 'email'],
            'owner.phone_number' => ['max:20', 'min:3'],
            'social_media.*' => ['array'],
            'social_media.*.review' => ['between:1,10'],
            'social_media.*.link' => ['required_with:social_media.*.review'],
            'owner.address.*' => ['array'],
            'owner.address.main_text' => ['min:3', 'max:50', 'max:250'],
            'owner.address.secondary_text' => ['min:3', 'max:50', 'max:250'],
            'owner.address.country' => ['min:2', 'max:50', 'regex:/^[a-zA-Z\s]+$/'],
            'owner.address.city' => ['min:2', 'max:50', 'regex:/^[a-zA-Z\s]+$/'],
            'owner.address.state' => ['min:2', 'max:50', 'regex:/^[a-zA-Z\s]+$/'],
            'owner.address.zip_code' => ['digits_between:1,20', 'numeric']
        ]);
//                return 22;

        //add image
        $imageName = null;
        if (array_key_exists('image', $validateData)) {
            $imageController = new ImageController();
            $imageName = $imageController->upload($this->request->image);
        }
        //add social media
        if (array_key_exists('social_media', $validateData)) {
            $media = new SocialMediaController();
            $media = $media->storeSocialMedia($this->clientId, $this->clientType, $this->request['social_media']);
            if ($media == 0)
                return $this->failedResponse('not found social media', Response::HTTP_NOT_FOUND);
        }
//        return 1;

        //add address
        $addressService = new AddressService;
        $address = $addressService->checkExist($this->clientType, $this->clientId);
        if ($address != null) {
            $addressUpdate = $addressService->update($validateData, $address->id);
        } else {
            $addressStore = $addressService->store($validateData, $this->clientType, $this->clientId);
        }

        //add owner
        if (array_key_exists('owner', $validateData)) {
            $ownerService = new OwnerService();
            $service = $ownerService->checkExists(Company::class, $this->clientId);
            if ($service == null)
                $owner = $ownerService->store($validateData['owner'], $this->clientType, $this->clientId);
            else {
                $owner = $ownerService->update($validateData['owner'], $service->id);
            }
        }
//        return 1;

        //add job
        $jobId = null;
        if (array_key_exists('job_title', $validateData)) {
            $jobRepository = new JobTitleRepository();
            $job = $jobRepository->findJob($this->request['job_title']);
            if ($job == null) {
                return failedResponse('Not Found job', Response::HTTP_NOT_FOUND);
            }
            $jobId = $job->toArray()['id'];
        }
//        return 1;

        //add information user
        $settingInformationRepository = new SettingInformationRepository();
        $company = $settingInformationRepository->updateInformationCompany($this->request, $this->clientId,
            $imageName, $jobId);

//        $this->clientEntity = auth($clientRole)->user();
//        $list = $this->clientEntity->load(['socialMedias', 'jobTitle']);
      $return=  $this->clientEntity::find( $this->clientId)->load(['jobTitle','address','owner']) ;
       $social= SocialMediable::query()->with('SocialMedia')->where('social_mediable_type',$this->clientType)->where('social_mediable_id',$this->clientId)->get();

        $return['social']=ShowProfileResource::collection($social);
        return successResponse(true, $return, 'send successfully');    }


    public function ListClient(){
       $client= Arr::collapse([ $this->pipelineWithoutPaginate(User::with('jobTitle'),[ \App\Filter\FilterClass\Users\Q::class]),$this->pipelineWithoutPaginate(Company::with('jobTitle'),[
          \App\Filter\FilterClass\Company\Q::class])]) ;
       return ($client)?$this->onSuccess(new PaginationResource($this->paginate($client),ListClientsResource::collection($this->paginate($client)))):$this->onSuccess($this->paginate([]));
    }
//    public function ListClient(){
//        $client= Arr::collapse([User::with('jobTitle')->get(),Company::with('jobTitle')->get()]) ;
//        return ($client)?$this->onSuccess(new PaginationResource($this->paginate($client),ListClientsResource::collection($this->paginate($client)))):$this->onSuccess([]);
//    }
    public function setInformation($clientRole)
    {
        $this->clientType = 'App\\Models\\' . Str::ucfirst($clientRole);
        $this->clientEntity = auth($clientRole)->user();
        $this->clientId = $this->clientEntity->id;
        $this->client = $this->clientType::find($this->clientId);
        return 1;
    }

    public function showInformationCompanyForOthers($clientRole,$id)
    {
        $this->setInformation($clientRole);

//        if ($clientRole)

//        return 1;
//        $list = $this->clientEntity->load(['socialMedias', 'jobTitle']);
        $return['company']= Company::query()->where('id', $id)->get()->load(['jobTitle','address','owner']) ;
//        return $return;
        $social= SocialMediable::query()->with('SocialMedia')->where('social_mediable_type',$this->clientType)->where('social_mediable_id',$this->clientId)->get();
        $return['social']=ShowProfileResource::collection($social);
        return successResponse(true, $return, 'send successfully');

//return successResponse(true, $list, 'send successfully');
    }
    public function showInformationUserForOthers(ShowInformationUserForOthersRequest $request)
    {
//        $this->setInformation($clientRole);
        $type='App\\Models\\' . Str::ucfirst($request->type);
        $id=$request->id;
//        return $id
       $clientEntity=  $type::find($request->id);
        if (!$type::find($request->id))
            return $this->onSuccess([]);
//        return $clientEntity;
//return $type::find($request->id);
//        return 1;
//        $list = $this->clientEntity->load(['socialMedias', 'jobTitle']);
        $return= $clientEntity::query()->withAvg('profileComments','rate')->withCount('participantsProject')->find($id)->load(['jobTitle'=>function($q){$q;}]) ;
        $social= SocialMediable::query()->with('SocialMedia')->where('social_mediable_type',$type)->where('social_mediable_id',$id)->get();

        $return['social']=ShowProfileResource::collection($social);
        $participants=    ParticipantsProject::query()->where('participants_project_type' ,$type)->where('participants_project_id',$id)
            ->pluck('project_id');
        $participantsCompany=ParticipantsProject::query()->whereIn('project_id',$participants)->where('participants_project_type' ,'App\Models\Company')->pluck('participants_project_id');
        $participantsUser=ParticipantsProject::query()->whereIn('project_id',$participants)->where('participants_project_type' ,User::class)->pluck('participants_project_id');
//        return $participants;

        $company= Company::query()->whereIn('id',$participantsCompany)->select('id','image','name')->get();
//        return $company;
        $user= User::query()->whereIn('id',$participantsUser)->select('id','image','name','last_name',)->get();
//        return Arr::collapse([$company,$user]);;
        $return['builders']=Arr::collapse([$company,$user]);
//        return GeneralContractor::whereIn('project_id',$participants)->get();
        $return['builders_not_register_general_contractor']=GeneralContractor::whereIn('project_id',$participants)->select('id','image','name')->get();
        $return['builders_not_owner']=Owner::whereIn('ownerable_id',$participants)->where('ownerable_type',Project::class)->select('id','first_name','last_name')->get();

        $grope=  Group::query()->with('members.client')
            ->whereHas('members',function ($q) use ($id,$type){
                $q->where('member_id',$id)->where('member_type',$type);
            }
            )->get();

        $return['group_image']=ShowOneProjectGropeResuorce::collection($grope);
        return successResponse(true, $return, 'send successfully');

//return successResponse(true, $list, 'send successfully');
    }
    public function searchUserByIndustry($id){
        $user= User::query()->where('job_title_id',$id)->select('email')->get();
        $company=Company::query()->where('job_title_id',$id)->select('email')->get();
       $client= Arr::collapse([$user,$company]);
        return (isset($client))? $this->onSuccess($client):$this->onSuccess([]);
    }


}







