<?php

namespace App\Http\Controllers;

use App\Filter\FilterClass\Map\Completed;
use App\Filter\FilterClass\Map\InProgress;
use App\Filter\FilterClass\Map\Priority;
use App\Filter\FilterClass\Map\Progress;
use App\Filter\FilterClass\Map\UpComing;
use App\Filter\FilterClass\project\Condition;
use App\Filter\FilterClass\project\ConditionCompany;
use App\Filter\FilterClass\project\ConditionUser;
use App\Filter\FilterClass\project\Industry;
use App\Filter\FilterClass\project\Name;
use App\Filter\FilterClass\project\participants\Q;
use App\Filter\FilterClass\project\ReferenceNumber;
use App\Filter\FilterClass\project\ZipCode;
use App\Http\Requests\AddMemberToProjectRequest;
use App\Http\Requests\Map\ShowMapRequests;
use App\Http\Requests\ProjectRequest;
use App\Http\Requests\ProjectUpdateRequest;
use App\Http\Requests\ShowAllProjectForOthersRequest;
use App\Http\Requests\ShowNotRegisterRequest;
use App\Http\Resources\ListProjectOnlyShowResource;
use App\Http\Resources\Map\ShowMapResource;
use App\Http\Resources\PaginationResource;
use App\Http\Resources\Project\ShowListProjectResource;
use App\Http\Resources\Project\ShowOneProjectResource;
use App\Http\Resources\SetProjectResource;
use App\Http\Resources\ShowNotRegisterResource;
use App\Models\Address;
use App\Models\Company;
use App\Models\GeneralContractor;
use App\Models\Image;
use App\Models\ImageKeyword;
use App\Models\Keyword;
use App\Models\Owner;
use App\Models\ParticipantsProject;
use App\Models\Project;
use App\Models\SpecialComment;
use App\Models\Style;
use App\Models\StyleImage;
use App\Models\User;
use App\Notifications\InvoiceNotification;
use App\Repositories\ProjectRepository;
use App\Repositories\Role\SearchRepository;
use App\Services\AddressService;
use App\Services\GeneralContractorService;
use App\Traits\ApiResponser;
use App\Traits\Helper;
use App\Traits\pipeline;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Route;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;
use Mojtaba\Chatable\Models\Member;


class ProjectController extends Controller
{
    use ApiResponser, pipeline, Helper;

    public $request;
    public $clientEntity;
    public $clientId;
    public $clientType;
    public ProjectRepository $projectRepository;

    public function __construct(Request $request)
    {
        $this->request = $request;
        $this->middleware('auth:sanctum');
    }

    public function store(EmailController $emailController, ProjectRequest $projectRequest, SearchRepository $searchRepository, $clientRole)
    {
        //find or add general contractor/company
        if ($this->request['project_general_contractor']) {
            $generaContractorService = new GeneralContractorService();
            $generalContractorFind = $generaContractorService->findByName($this->request['project_general_contractor']);
            if ($generalContractorFind->toArray() == null) {
                $generalContractor = $generaContractorService->store($this->request['project_general_contractor']);
                $projectableId = $generalContractor->toArray()['id'];
            } else {
                $projectableId = $generalContractorFind->toArray()[0]['id'];
            }
            $projectableType = 'App\Models\GeneralContractor';
        } elseif ($this->request['project_company']) {
            $companyFind = Company::where('name', $this->request['project_company.name'])->get()->toArray();
            if ($companyFind == null) {
                return $this->successResponse(false, null, 'not found this company');
            }
            $projectableId = $companyFind[0]['id'];
            $projectableType = 'App\Models\Company';
        } else {
            return $this->successResponse(false, null, 'enter general contractor');
        }

        //set reference number
        randomNumber:
        $referenceNumber = rand(100000000, 999999999);
        $result = Project::where('reference_number', $referenceNumber)->get();
        if ($result->toArray() != null) {
            goto randomNumber;
        }

        //add project to DB
        $project = Project::create([
            'name' => $this->request['project_name'],
            'type' => $this->request['project_type'],
            'condition' => $this->request['project_condition'],
            'priority' => $this->request['project_priority'],
            'reference_number' => $referenceNumber,
            'projectable_type' => $projectableType,
//            'style' => $this->request['project_style'],
            'projectable_id' => $projectableId,
            'start_date' => $this->request['start_date'],
            'end_date' => $this->request['end_date'],
        ]);

        $style = Style::create(['name' => $this->request['project_style']]);
        if ($project && $style)
            StyleProject::create(['project_id' => $project->id, 'style_id' => $style->id]);
        $projectId = $project->toArray()['id'];

        //add image and upload
        $imageName = null;

        if ($this->request['image']) {
            $imageController = new ImageController();

            foreach ($this->request['image'] as $keys => $images) {
                //dd($this->request['image']);
                //dd($images);
                $imageName = $imageController->move($images['name']);
                if (!isset($images['link'])) {
                    $images['link'] = null;
                }
                $insertImage = $imageController->insertImage($images['name'], "App\Models\Project", 1, $images['link']);
                $imageIds[$keys] = $insertImage->toArray()['id'];
                if (isset($images['tag'])) {
                    foreach ($images['tag'] as $key => $tag) {
                        $keywordIds[$keys][] = $imageController->insertKeywordImage($images, $tag);
                    }
                } else {
                    $keywordIds[$keys][] = null;
                }
                //$insertImageProject = $imageController->insertProjectImage($insertImage->toArray()['id'], $projectId);
            }
            foreach ($imageIds as $num => $imageId) {
                //dd($keywordIds[3][0]);
                //dd($keywordIds);
                $photo = Image::find($imageId);
                if ($keywordIds[$num][0] == null) {
                    $photo->keywords()->detach();
                } else {
                    $photo->keywords()->sync($keywordIds[$num]);
                }
            }
        }

        //add address
        if ($this->request['project_address']) {
            $addressService = new AddressService();
            $address = $addressService->store($this->request, Project::class, $project->id);
//            $addressController = new AddressController();
//            $address = $addressController->store($this->request['project_address'], 'App\Models\Project', $projectId);
        }

        //add owner
        if ($this->request['project_owner']) {
            $ownerController = new OwnerController();
            $owner = $ownerController->store($this->request['project_owner'], 'App\Models\Project', $projectId);
        }

        //$emailUser = auth('company')->user()->toArray()['email'];
        $emailUser = 'company@gmail.com';
        $emailUser = auth($clientRole)->user()->email;
        //add message
        if ($this->request['project_owner.message'] && $this->request['project_owner.email']) {
            if ($this->request['owner.messageEmail'] == true) {
                $emailController->sendEmailMessage($this->request['project_owner.email'], $emailUser, $this->request['owner.message']);
            }
            $messageController = new MessageController();
            $message = $messageController->store($this->request['owner.message'], 'App\Models\owner', $owner->toarray()['id']);
        }

        if ($this->request['project_generalContractor.message'] && $this->request['project_generalContractor.email']) {
            if ($this->request['project_generalContractor.message.notify'] == true) {
                $emailController->sendEmailMessage($this->request['project_generalContractor.email'], $emailUser, $this->request['project_generalContractor.message.text']);
            }

            $messageController = new MessageController();
            $message = $messageController->store($this->request['project_generalContractor.message'], 'App\Models\GeneralContractor', $projectableId);
        }

        //add user to project
        //$emailUser = auth('company')->user()->toArray()['email'];
        $emailUser = 'company@gmail.com';
        $data = $searchRepository->searchEmail('\App\Models\Company', $emailUser);
        $dataId = $data->toArray()['id'];
        $dataId = 1;
        $table = DB::table('participant_project')->insert([
            'participant_project_type' => '\App\Models\Company',
            'participant_project_id' => $dataId,
            'project_id' => $projectId
        ]);

        return $this->successResponse(true, null, 'add successfully');
    }

    public function testAddress(AddressService $addressService)
    {
        $a = $addressService->checkValidAddress($this->request['country'], $this->request['state'], $this->request['city']);
        dd($a);
    }

    public function findByType($priority)
    {

    }

    public function findByAddress()
    {
        Project::select('id', 'name');
    }

    public function findByCondition($condition)
    {
        Project::select('id', 'name', 'condition')->where('condition', $condition)->get();
    }

    public function addUserToProject()
    {
        //hard code
        $participantProjectsType = 'App\Models\User';
        $participantProjectsId = 1;

        $table = DB::table('participant_project')->where('id', $this->request['project_id'])->update([
            'participant_project_type' => '\App\Models\Company',
            'participant_project_id' => $participantProjectsId,
            'project_id' => Request()->project_id,
        ]);

        if ($this->request['image']) {
            $imageController = new ImageController();

            foreach ($this->request['image'] as $keys => $images) {
                //dd($this->request['image']);
                //dd($images);
                $imageName = $imageController->move($images['name']);
                if (!isset($images['link'])) {
                    $images['link'] = null;
                }
                $insertImage = $imageController->insertImage($images['name'], "App\Models\Project", 1, $images['link']);
                $imageIds[$keys] = $insertImage->toArray()['id'];
                if (isset($images['tag'])) {
                    foreach ($images['tag'] as $key => $tag) {
                        $keywordIds[$keys][] = $imageController->insertKeywordImage($images, $tag);
                    }
                } else {
                    $keywordIds[$keys][] = null;
                }
                //$insertImageProject = $imageController->insertProjectImage($insertImage->toArray()['id'], $projectId);
            }
            foreach ($imageIds as $num => $imageId) {
                //dd($keywordIds[3][0]);
                //dd($keywordIds);
                $photo = Image::find($imageId);
                if ($keywordIds[$num][0] == null) {
                    $photo->keywords()->detach();
                } else {
                    $photo->keywords()->sync($keywordIds[$num]);
                }
            }
        }
        return 1;
    }

    public function findByClient($clientRole)
    {
        $clientEntity = auth($clientRole)->user();
        $clientId = auth($clientRole)->user()->id;
        $clientType = 'App\\Models\\' . Str::ucfirst($clientRole);

        $client = $clientType::where('id', $clientId)->with('projects')->with('projects.owner')->get();
        $list = DB::table('participants_project')->where('project_id', 11)->get();
        return successResponse(true, [$client, $list], 'send successfully');
    }

    public function addClientToProject($id, $clientRole)
    {
        //get project id + user + job title + start date + end date + image with keyword with link
        //add image
        //$imageName = null;
        $id = $this->request['project_id'];
        $this->clientEntity = auth($clientRole)->user();
//        return  $clientRole;
        $this->clientId = $this->clientEntity->id;
        $this->clientType = 'App\\Models\\' . Str::ucfirst($clientRole);


        if ($this->request['image']) {
            $imageController = new ImageController();

            foreach ($this->request['image'] as $keys => $images) {
                //dd($this->request['image']);
                //dd($images);
                $imageName = $imageController->move($images['name']);
                if (!isset($images['link'])) {
                    $images['link'] = null;
                }
                $insertImage = $imageController->insertImage($images['name'], Project::class, $id, $this->clientType, $this->clientId, $images['link']);
                $imageIds[$keys] = $insertImage->toArray()['id'];
                if (isset($images['tag'])) {
                    foreach ($images['tag'] as $key => $tag) {
                        $keywordIds[$keys][] = $imageController->insertKeywordImage($images, $tag);
                    }
                } else {
                    $keywordIds[$keys][] = null;
                }
                //$insertImageProject = $imageController->insertProjectImage($insertImage->toArray()['id'], $projectId);
            }
//            return 1;
            foreach ($imageIds as $num => $imageId) {
                //dd($keywordIds[3][0]);
                //dd($keywordIds);
                $photo = Image::find($imageId);
                if ($keywordIds[$num][0] == null) {
                    $photo->keywords()->detach();
                } else {
                    $photo->keywords()->sync($keywordIds[$num]);
                }
            }
        }

//return $this->request['job_title_id'];
        //add user to project
        //$emailUser = auth('company')->user()->toArray()['email'];
        $clientEmail = $this->clientEntity->email;
        $searchRepository = new SearchRepository();
        $data = $searchRepository->searchEmail($this->clientType, $clientEmail);
        //$dataId = $data->toArray()['id'];
        $table = DB::table('participants_project')
            ->insert([
                'participant_project_type' => $this->clientType,
                'participant_project_id' => $this->clientId,
                'project_id' => $id,
                'job_title_id' => $this->request['job_title_id'],
                'start_date' => '2020-2-1'
            ]);
        $route = Route::current()->getName();
        if ($route == 'store') {
            return 1;
        }
        return $this->successResponse(true, null, 'add client to project successfully');
    }

    public function addClientToProject2($id, $clientRole, $projectRequest, $owner, $generalContractor)
    {
        //get project id + user + job title + start date + end date + image with keyword with link
        //add image
        //$imageName = null;
        $id = $this->request['project_id'];
        $this->clientEntity = auth($clientRole)->user();
//        return  $clientRole;
        $this->clientId = $this->clientEntity->id;
        $this->clientType = 'App\\Models\\' . Str::ucfirst($clientRole);

//        return 2;
//return $id;
//        return 3;
        $specialComment = new SpecialCommentController();

        if ($this->request['project_general_contractor.text'] && $this->request['project_general_contractor.rate']) {
//                return 1;
            if ($this->request['project_general_contractor.id_general'] && $this->request['project_general_contractor.type']) {
//                return 2;
                $specialComment->store($this->request['project_general_contractor'], $id, $this->clientType, $this->clientId, $this->request['project_general_contractor.id_general'], $this->getPathClass($this->request['project_general_contractor.type']));
            } else {
//                return $generalContractor;
                if ($generalContractor)
                    $specialComment->store($this->request['project_general_contractor'], $id, $this->clientType, $this->clientId, $generalContractor->id, $generalContractor->type);
            }
        }

        if ($this->request['project_owner.text'] && $this->request['project_owner.rate']) {
//                return 1;
            if ($this->request['project_owner.id'] && $this->request['project_owner.type']) {
//                return 2;
                $specialComment->store($this->request['project_owner'], $id, $this->clientType, $this->clientId, $this->request['project_owner.id'], $this->getPathClass($this->request['project_owner.type']));
            } else {
//                return $generalContractor;

                $specialComment->store($this->request['project_owner'], $id, $this->clientType, $this->clientId, $owner->id, $owner->type);
            }
        }

        ///////////////////////////
//return $projectRequest;
        if ($this->request['image']) {
//            return $this->request['image'];
            if (is_array($this->request['image'])) {

                $imageController = new ImageController();

            foreach ($this->request['image'] as $keys => $images) {
                //dd($this->request['image']);
                //dd($images);
//                return $images['style'];

                $imageName = $imageController->move($images['name']);
                if (!isset($images['link'])) {
                    $images['link'] = null;
                }
                $insertImage = $imageController->insertImage($images['name'], Project::class, $id, $this->clientType, $this->clientId, $images['link']);
                $imageIds[$keys] = $insertImage->toArray()['id'];
//                return $imageIds[$keys] ;
                if (isset($images['tag'])) {
                    foreach ($images['tag'] as $key => $tag) {
                        $keywordIds[$keys][] = $imageController->insertKeywordImage($images, $tag);
                    }
                } else {
                    $keywordIds[$keys][] = null;
                }


                if (isset($images['style'])) {
//                    $title=
//                    $style = Style::create(['title' => ]);
                    if (isset($imageIds[$keys]))
                        StyleImage::create(['image_id' => $imageIds[$keys], 'style_id' => $images['style']]);
                }
                //$insertImageProject = $imageController->insertProjectImage($insertImage->toArray()['id'], $projectId);
            }
//            return 1;
            foreach ($imageIds as $num => $imageId) {
                //dd($keywordIds[3][0]);
                //dd($keywordIds);
                $photo = Image::find($imageId);
                if ($photo) {
                    if ($keywordIds[$num][0] == null) {
                        $photo->keywords()->detach();
                    } else {
                        $photo->keywords()->sync($keywordIds[$num]);
                    }
                }
            }
        }else{
                return $this->onFaile('invalid photo ');

            }
        }

        //////////////////////////////////////////////////

// return $this->request['job_title_id'];
        //add user to project
        //$emailUser = auth('company')->user()->toArray()['email'];
        $clientEmail = $this->clientEntity->email;
        $searchRepository = new SearchRepository();
        $data = $searchRepository->searchEmail($this->clientType, $clientEmail);
        //$dataId = $data->toArray()['id'];
        $table = DB::table('participants_project')->insert([
            'participants_project_type' => $this->clientType,
            'participants_project_id' => $this->clientId,
            'project_id' => $id,
            'role' => ParticipantsProject::CREATORE_ROLE,
//            'job_title_id' => $this->request['job_title_id'],
            'start_date' => $this->request['start_date'],
        ]);
        $route = Route::current()->getName();
        if ($route == 'store') {
            return 1;
        }
        return $this->successResponse(true, null, 'add client to project successfully');
    }

    public function storeProject(EmailController $emailController, ProjectRequest $projectRequest, SearchRepository $searchRepository, $clientRole)
    {
//        Log::info($this->request);
//        return $this->request['image'];
//        $imageController = new ImageController();
//        $imageController->checkImageForProject($this->request['image']);
        ///////////////////////////
//return $this->request;
        $this->setInformation($clientRole);
//        return 1;

//        foreach ($this->request['image'] as $keys => $images) {
//            $image = Storage::exists('/temp/images/' . $images['name']);
//            if (!$image)
//                return $this->onFaile('photo not found');
//        }
        if ($this->request['project_general_contractor.id_general']) {
            $client=$this->clientType::find($this->clientId);
            if ($client->job_title_id!=1)
                return $this->onFaile('The job title of the contracting industry is not public');
        }


//        return $projectRequest->image;
        if ($projectRequest->image) {
            if (is_array($projectRequest->image)) {
                foreach ($this->request['image'] as $keys => $images) {
                    $image = Storage::exists('/temp/images/' . $images['name']);
                    if (!$image)
                        return $this->onFaile(' photo not found');
                }
            } else {
                return $this->onFaile('invalid photo ');
            }
        }

        if ($projectRequest['project_general_contractor.image']) {
            $image = Storage::exists('/temp/images/' . $projectRequest['project_general_contractor.image']);
            if (!$image)
                return $this->onFaile('general contractor photo not find ');
        }

//        return $this->request['project_owner.email'];
//        return 1;


        if ($this->request['project_general_contractor.id_general'] && $this->request['project_general_contractor.type']) {
            $contractor = ('App\\Models\\' . Str::ucfirst($this->request['project_general_contractor.type']))::find($this->request['project_general_contractor.id_general']);
//            return $contractor;
            if ($contractor == null)
                return $this->onFaile('general contractor not find ');

        } elseif (!$this->request['project_general_contractor.name'] || !$this->request['project_general_contractor.phone_number'] || !$this->request['project_general_contractor.email'] && !$this->request['project_general_contractor.email']) {
            return $this->onFaile('Complete General Contractor information');
        }
        $style = Style::where('id', $this->request['project_style_id'])->first();
//        return $style;
        if (!($style))
            return $this->onFaile('style not found');
//        return $projectRequest['project_general_contractor.image'] ;

        if ($projectRequest['project_general_contractor.image']) {
            $image = Storage::exists('/temp/images/' . $projectRequest['project_general_contractor.image']);
//                return  $image;
            if (!$image)
                return $this->onFaile(' general contractor photo  not found ');

        }

        //find or add general contractor/company

//        if ($this->request['project_general_contractor']) {
//            $generaContractorService = new GeneralContractorService();
//            $generalContractorFind = $generaContractorService->findByName($this->request['project_general_contractor']);
//
//            if ($generalContractorFind->toArray() == null) {
//                $generalContractor = $generaContractorService->store($this->request['project_general_contractor']);
//                $projectableId = $generalContractor->toArray()['id'];
//            } else {
//                $projectableId = $generalContractorFind->toArray()[0]['id'];
//            }
//            $projectableType = 'App\Models\GeneralContractor';
//        } elseif ($this->request['project_company']) {
//            $companyFind = Company::where('name', $this->request['project_company.name'])->get()->toArray();
//            if ($companyFind == null) {
//                return $this->successResponse(false, null, 'not found this name company');
//            }
//            $projectableId = $companyFind[0]['id'];
//            $projectableType = 'App\Models\Company';
//        } else {
//            return $this->successResponse(false, null, 'enter general contractor');
//        }

        //set reference number
        randomNumber:
        $referenceNumber = rand(100000000, 999999999);
        $result = Project::where('reference_number', $referenceNumber)->get();
        if ($result->toArray() != null) {
            goto randomNumber;
        }
//        project_owner=[
//            address: {
//                 city:{
//                  }
//                 }
//            ];
        //add project to DB


        $project = Project::create([
            'name' => $this->request['project_name'],
            'type' => $this->request['project_type'],
            'condition' => $this->request['project_condition'],
            'priority' => $this->request['project_priority'],
            'about_project' => $this->request['about_project'],
            'reference_number' => $referenceNumber,
            'projectable_type' => $this->clientType,
            'style' => $this->request['project_style_id'],
            'projectable_id' => $this->clientId,
            'start_date' => $this->request['project_start'],
            'end_date' => $this->request['project_end'],
        ]);

        $projectId = $project->toArray()['id'];
        $this->request['project_id'] = $projectId;
        //add image and upload


        //add address
        $addressService = new AddressService();
        $address = $addressService->store($this->request['project_address'], Project::class, $project->id);


        $generalContactor = null;
        if ($this->request['project_general_contractor.id_general']) {
            $client=$this->clientType::find($this->clientId);
            if ($client->job_title_id!=1)

            $table = DB::table('participants_project')->insert([
                'participants_project_type' => $this->getPathClass($this->request['project_general_contractor.type']),
                'participants_project_id' => $this->request['project_general_contractor.id_general'],
                'project_id' => $project->id,
                'role' => ParticipantsProject::GENERAL_CONTRACTOR_ROLE,

//            'job_title_id' => $this->request['job_title_id'],
                'start_date' => $this->request['start_date'],
                'end_date' => $this->request['start_date'],
            ]);
//            return 3;
//          return  $this->request['project_general_contractor.id_general'];
        } else {
            $general = new  GeneralContractorService();
            $generalContactor = $general->storeUpdate($projectRequest['project_general_contractor'], $project->id);
//            return $generalContactor;
            $generalContactor['type'] = GeneralContractor::class;
        }

        //add owner
        $owner = null;
        if ($this->request['project_owner']) {
            if ($this->request['project_owner.id'] && $this->request['project_owner.type']) {
//                return 50;
                $table = DB::table('participants_project')->insert([
                    'participants_project_type' => $this->getPathClass($this->request['project_owner.type']),
                    'participants_project_id' => $this->request['project_owner.id'],
                    'project_id' => $project->id,
//            'job_title_id' => $this->request['job_title_id'],
                    'role' => ParticipantsProject::OWNER_ROLE,
                    'start_date' => $this->request['start_date'],
                    'end_date' => $this->request['end_date'],
                ]);

            } else {
                $ownerController = new OwnerController();
                $owner = $ownerController->store($this->request['project_owner'], 'App\Models\Project', $projectId);
                $owner['type'] = Owner::class;
            }
        }
//        $emailUser = auth('company')->user()->toArray()['email'];
//        $emailUser = 'company@gmail.com';
//        $emailUser = auth($clientRole)->user()->email;
//        add message

        if ($this->request['project_owner.is_send']) {
            if ($this->request['project_owner.text'] && $this->request['project_owner.email']) {
//                if ($this->request['owner.messageEmail'] == true) {
                $emailController->sendEmailMessage($this->request['project_owner.email'], env('FROM_EMAIL'), $this->request['owner.message']);
//                }
                if ($this->request['project_owner.id'] && $this->request['project_owner.type']) {
                    $client = ($this->getPathClass($this->request['project_owner.type']))::find($this->request['project_owner.id']);
                    $client->notify(new InvoiceNotification($this->notificationData($client, 'You have been selected as the owner of the selected project' . $project->name, 'Added to new project')));
                }
//            $messageController = new MessageController();
//            $message = $messageController->store($this->request['owner.message'], 'App\Models\owner', $owner->toarray()['id']);
            }
        }
        if ($this->request['project_general_contractor.is_send']) {

            if ($this->request['project_generalContractor.text'] && $this->request['project_generalContractor.email']) {
//                if ($this->request['project_generalContractor.message.notify'] == true) {
                $emailController->sendEmailMessage($this->request['project_general_contractor.email'], env('FROM_EMAIL'), $this->request['project_general_contractor.text']);
//                }
                if ($this->request['project_general_contractor.id_general'] && $this->request['project_general_contractor.type']) {
                    $client = ($this->getPathClass($this->request['project_general_contractor.type']))::find($this->request['project_general_contractor.id_general']);
                    if ($client) {
                        $client->notify(new InvoiceNotification($this->notificationData($client, 'You have been selected as the generalContractor of the selected project' . $project->name, 'Added to new project')));
                    }
                }
//
//                $messageController = new MessageController();
//                $message = $messageController->store($this->request['project_generalContractor.message'], 'App\Models\GeneralContractor', $projectableId);
            }
//  $clientRole::find($projectId);
        }
//call method that add client
//        return 1;

        $this->addClientToProject2($projectId, $clientRole, $projectRequest->validated(), $owner, $generalContactor);

//        return $this->successResponse(true, Project::where('id', $project->id)->with('address', 'images')->first(), 'add successfully');
        return $this->successResponse(true, SetProjectResource::make(Project::where('id', $projectId)
            ->with(['address', 'images', 'owner', 'generalContractor',
                'specialComments' => function ($q) {
//                ->whereHas('client',function ($q){
                    $q->where('client_type', $this->clientType)->where('client_id', $this->clientId);
//        });
                }])->first()), 'add successfully');
    }

    public function findProject($id)
    {
//        return User::with('clientComments')->get();
        $project = Project::where('id', $id)
            ->with(['specialComments.profile',
                'owner' => function ($q) {
                    $q->withAvg('specialCommentsProfile', 'rate');
                }, 'generalContractor' => function ($q) {
                    $q->withAvg('specialCommentsProfile', 'rate');
                },
                'specialComments.client.images',
                'participants.participantsProject.jobTitle',
                'participants.industry', 'images', 'images.style',
                'images.keywords', 'address', 'participants.participantsProject' => function ($q) {
                    $q->withAvg('clientComments', 'rate')->with('participantsProject');
                }])->first();
//        $project['rrrrrrrrrrrr']=1;
        return ($project) ? $this->onSuccess(new ShowOneProjectResource($project)) : $this->onFaile('project not found');
    }

    public function showProjectMap($clientRole)
    {
//        return 1;
//return ParticipantsProject::all();
        $clientEntity = auth($clientRole)->user();
        $clientId = auth($clientRole)->user()->id;
        $clientType = 'App\\Models\\' . Str::ucfirst($clientRole);
//return 11;
        $type = $clientRole;
//        return \request()->id;
//return   (\request()->has('id')) ;
        $map = null;
        if (\request()->type == 'user' || \request()->type == 'company') {
//            return 5;
            if (\request()->has('id')) {
//                return 3;
                $map = $this->pipelineWithoutPaginate(Project::query()->with('address')
                    ->whereHas('participants', function ($q) {
                        $q->where('participants_project_type', \request()->type)
                            ->where('participants_project_id', \request()->id);
                    }), [UpComing::class, InProgress::class, Completed::class,ZipCode::class,Priority::class]);

            } else {
                if (auth($type)->user()) {
                    $map = $this->pipelineWithoutPaginate(Project::query()->with('address')
                        ->whereHas('participants', function ($q) use ($type) {
                            $q->where('participants_project_type', 'App\\Models\\' . Str::ucfirst($type))
                                ->where('participants_project_id', auth($type)->user()->id);
                        }), [UpComing::class, InProgress::class, Completed::class,ZipCode::class,Priority::class]);
                } else {
                    return $this->onSuccess([]);
                }
            }
        } else {
//            return 6;
            $map = $this->pipelineWithoutPaginate(Project::query()->with('address')
                , [UpComing::class, InProgress::class, Completed::class ,ZipCode::class,Priority::class,Q::class,\App\Filter\FilterClass\project\Address::class
                ,Industry::class]);
//            return $map;
        }
//        if (\request()->type == 'company')
//            $map = $this->pipelineWithoutPaginate(Project::query()->with('address'), [ConditionCompany::class,UpComing::class,InProgress::class,Completed::class]);

        return ($map) ? $this->onSuccess(ShowMapResource::collection($map)) : $this->onSuccess([]);
    }

    public function showtMap($clientRole)
    {
//return ParticipantsProject::all();
        $clientEntity = auth($clientRole)->user();
        $clientId = auth($clientRole)->user()->id;
        $clientType = 'App\\Models\\' . Str::ucfirst($clientRole);

        $map = null;
        if ($clientRole == 'user')
            $map = $this->pipelineWithoutPaginate(Project::query()->with('address'), [ConditionUser::class]);
        if ($clientRole == 'company')
            $map = $this->pipelineWithoutPaginate(Project::query()->with('address'), [ConditionCompany::class]);

        return ($map) ? $this->onSuccess($map) : $this->onFaile([]);
    }

    public function showAllProject($clientRole)
    {
//        return $clientRole;
//        return ParticipantsProject::with('client')->get();
        $clientEntity = auth($clientRole)->user();
        $clientId = auth($clientRole)->user()->id;
        $clientType = 'App\\Models\\' . Str::ucfirst($clientRole);
//        return $clientEntity;
        $project =
            $this->pipelineWithoutPaginate(Project::query()->whereHas('participants', function ($q) use ($clientId, $clientType) {
                $q->where('participants_project_type', $clientType)->where('participants_project_id', $clientId);
            })->withCount('images')
                ->with(['participants.industry', 'participants.participantsProject', 'images.keywords'])
                , [Condition::class]);
//        return ShowListProjectResource::collection($project);
//            $this->pipeline(
//                Project::query()->with(['participants.industry','participants.participantsProject','images.keywords'
////            =>function(){
////            }
//            ,'address'])
//            ,[ConditionCompany::class]);
//         $project['role']=55555555555555555;
//        return 1;
//        return $project;
        return ($project) ? $this->onSuccess(new PaginationResource($this->paginate($project), ShowListProjectResource::collection(($project)))) : $this->onSuccess($this->paginate([]));
    }

    public function UpdateProject($clientRole, ProjectUpdateRequest $projectRequest, $id)
    {
//       return $projectRequest->image;


        if ($projectRequest->image) {
            if (is_array($projectRequest->image)) {
                foreach ($this->request['image'] as $keys => $images) {
                    $image = Storage::exists('/temp/images/' . $images['name']);
                    if (!$image)
                        return $this->onFaile('photo not found ');
                }
            } else {
                return $this->onFaile('invalid photo ');
            }
        }
        if ($projectRequest['project_general_contractor.image']) {
            $image = Storage::exists('/temp/images/' . $projectRequest['project_general_contractor.image']);
            if (!$image)
                return $this->onFaile(' general contractor photo not find');

        }

//        return $projectRequest;

        $project = Project::where('id', $id)->first();
        if (!$project)
            return $this->onFaile('project not found');
        $this->setInformation($clientRole);

        if ($this->request['project_general_contractor.id_general']) {
            $client=$this->clientType::find($projectRequest['project_general_contractor.id_general']);
            if ($client->job_title_id!=1)
                return $this->onFaile('The job title of the contracting industry is not public');
        }

        $client = ParticipantsProject::where('participants_project_id', $this->clientId)->where('participants_project_type', $this->clientType)->where('project_id', $id)->first();
        if ($client)
            if ($client->role != ParticipantsProject::CREATORE_ROLE)
                return $this->onFaile('access denied', 403);
//        if ($this->request['image']) {
//            $images = Image::where('imageable_type', Project::class)->where('imageable_id', $id)->get();
//          return $images;
//            if (($images)->isNotEmpty()) {
////                return 1;
//                foreach ($images as $image) {
////                    return $image;
//                    if (Storage::exists('public/images/' . $image->name))
//                        Storage::delete('public/images/' . $image->name);
//                $image->delete();
//                    $keyword =ImageKeyword::where('image_id',$image->id)->delete();
//                }
//
//            }
//            return $projectRequest['project_address'];
        if ($projectRequest['project_address']) {
            Address::where('addressable_id', $id)->where('addressable_type', Project::class)->delete();
            $addressService = new AddressService();
            $address = $addressService->store($projectRequest['project_address'], Project::class, $id);
        }

        $project->update([
            'name' => $projectRequest['project_name'] ?? null,
            'type' => $projectRequest['project_type'] ?? null,
            'condition' => $projectRequest['project_condition'] ?? null,
            'priority' => $projectRequest['project_priority'] ?? null,
            'style' => $this->request['project_style_id'] ?? null,
            'start_date' => $this->request['project_start'] ?? null,
            'end_date' => $this->request['project_end'] ?? null,
        ]);
//                        return  is_array($projectRequest->image);
        if ($projectRequest->image && is_array($projectRequest->image)) {
            $this->addPhotoToProject($id);
        }
//        return 2;

        if ($projectRequest->project_general_contractor) {
            $clientProject = ParticipantsProject::query()
                ->where('project_id', $id)
                ->where('role', ParticipantsProject::GENERAL_CONTRACTOR_ROLE)->first();
//           return $clientProject;
//            ParticipantsProject::query()
//                ->where('project_id', $id)
//                ->where('role', ParticipantsProject::GENERAL_CONTRACTOR_ROLE)
//                ->delete();
            $GeneralContractorList = GeneralContractor::query()
                ->where('project_id', $id)->pluck('id');
            GeneralContractor::query()
                ->where('project_id', $id)->delete();
//            return 10;
            $specialComment = new SpecialCommentController();
//return $clientProject;
            if ($this->request['project_general_contractor.id_general'] && $this->request['project_general_contractor.type']) {
                if ($clientProject)
                    SpecialComment::where('project_id', $id)->where('profile_type', $clientProject->participants_project_type)->where('profile_id', $clientProject->participants_project_id)->delete();

                $table = DB::table('participants_project')->insert([
                    'participants_project_type' => $this->getPathClass($this->request['project_general_contractor.type']),
                    'participants_project_id' => $this->request['project_general_contractor.id_general'],
                    'project_id' => $project->id,
                    'role' => ParticipantsProject::GENERAL_CONTRACTOR_ROLE,

//            'job_title_id' => $this->request['job_title_id'],
                    'start_date' => $this->request['start_date'],
                    'end_date' => $this->request['start_date'],
                ]);
//            return 3;
//          return  $this->request['project_general_contractor.id_general'];
            } else {

//              return $owners;
//return $GeneralContractorList;
//                GeneralContractor::query()
//                    ->where('project_id', $id)->delete();
//                return 4;
                $general = new  GeneralContractorService();
                $generalContactor = $general->storeUpdate($projectRequest['project_general_contractor'], $id);
                $generalContactor['type'] = GeneralContractor::class;


                if ($this->request['project_general_contractor.text'] && $this->request['project_general_contractor.rate']) {
//                    SpecialComment::where('project_id',$id)->whereIn('profile_id',$owners)->where('profile_type',Owner::class)->delete();
                    SpecialComment::where('project_id', $id)->whereIn('profile_id', $GeneralContractorList)->where('profile_type', GeneralContractor::class)->delete();

//                return 1;
                    if ($this->request['project_general_contractor.id_general'] && $this->request['project_general_contractor.type']) {
//                return 2;
                        $specialComment->store($this->request['project_general_contractor'], $id, $this->clientType, $this->clientId, $this->request['project_general_contractor.id_general'], $this->getPathClass($this->request['project_general_contractor.type']));
                    } else {
//                return $generalContractor;
                        if ($generalContactor)
                            $specialComment->store($this->request['project_general_contractor'], $id, $this->clientType, $this->clientId, $generalContactor->id, $generalContactor->type);
                    }
                }
            }
        }
//                return 2;

        if ($projectRequest->project_owner) {
            ParticipantsProject::query()
                ->where('project_id', $id)
                ->where('role', ParticipantsProject::OWNER_ROLE)
                ->delete();
            if ($projectRequest['project_owner.id'] && $projectRequest['project_owner.type']) {
//                return 50;
                $table = DB::table('participants_project')->insert([
                    'participants_project_type' => $this->getPathClass($this->request['project_owner.type']),
                    'participants_project_id' => $this->request['project_owner.id'],
                    'project_id' => $project->id,
//            'job_title_id' => $this->request['job_title_id'],
                    'role' => ParticipantsProject::OWNER_ROLE,
                    'start_date' => $this->request['start_date'],
                    'end_date' => $this->request['end_date'],
                ]);

            } else {
                $owners = Owner::query()
                    ->where('ownerable_id', $id)->where('ownerable_type', Project::class)->pluck('id');
//              return $owners;
                SpecialComment::where('project_id', $id)->whereIn('profile_id', $owners)->where('profile_type', Owner::class)->delete();

                Owner::query()
                    ->where('ownerable_id', $id)->where('ownerable_type', Project::class)->delete();
//                return $owners;


                $ownerController = new OwnerController();
                $owner = $ownerController->store($projectRequest['project_owner'], 'App\Models\Project', $id);
                $owner['type'] = Owner::class;


                if ($this->request['project_owner.text'] && $this->request['project_owner.rate']) {
//                return 1;
//                    if ($this->request['project_owner.id'] && $this->request['project_owner.type']) {
//                return 2;
//                        $specialComment->store($this->request['project_owner'], $id, $this->clientType, $this->clientId, $this->request['project_owner.id'], $this->getPathClass($this->request['project_owner.type']));
//                    } else {
//                return $generalContractor;
//                        SpecialComment::where('client_id',$this->clientId)->where('client_type',$this->clientType)->where('profile_type',$this->clientType)->
                    $specialComment->store($this->request['project_owner'], $id, $this->clientType, $this->clientId, $owner->id, $owner->type);
//                    }
                }

            }
        }


//return 5555;

        return $this->successResponse(true, SetProjectResource::make(Project::where('id', $id)->with(['address', 'images', 'owner', 'generalContractor',
            'specialComments' => function ($q) {
//                ->whereHas('client',function ($q){
                $q->where('client_type', $this->clientType)->where('client_id', $this->clientId);
//        });
            }])->first()), 'add successfully');

//            return 1;
//          return   Image::where('imageable_type',Project::class)->where('imageable_id',$id)->get();
//        }


    }

    public function addPhotoToProject($id)
    {
        if ($this->request['image']) {
            $imageController = new ImageController();

            foreach ($this->request['image'] as $keys => $images) {
                //dd($this->request['image']);
                //dd($images);
//                return $images['style'];

                $imageName = $imageController->move($images['name']);
                if (!isset($images['link'])) {
                    $images['link'] = null;
                }
                $insertImage = $imageController->insertImage($images['name'], Project::class, $id, $this->clientType, $this->clientId, $images['link']);
                $imageIds[$keys] = $insertImage->toArray()['id'];
//                return $imageIds[$keys] ;
                if (isset($images['tag'])) {
                    foreach ($images['tag'] as $key => $tag) {
                        $keywordIds[$keys][] = $imageController->insertKeywordImage($images, $tag);
                    }
                } else {
                    $keywordIds[$keys][] = null;
                }


                if (isset($images['style'])) {
//                    $title=
//                    $style = Style::create(['title' => ]);
                    if (isset($imageIds[$keys]))
                        StyleImage::create(['image_id' => $imageIds[$keys], 'style_id' => $images['style']]);
                }
                //$insertImageProject = $imageController->insertProjectImage($insertImage->toArray()['id'], $projectId);
            }
//            return 1;
            foreach ($imageIds as $num => $imageId) {
                //dd($keywordIds[3][0]);
                //dd($keywordIds);
                $photo = Image::find($imageId);
                if ($photo) {
                    if ($keywordIds[$num][0] == null) {
                        $photo->keywords()->detach();
                    } else {
                        $photo->keywords()->sync($keywordIds[$num]);
                    }
                }
            }
        }

    }

    public function deletePhotoToProject($clientRole, $id)
    {
//        return $id;
        $image = Image::where('id', $id)->first();
//          return $image;
        if ($image) {
//                return 1;
//                    return $image;
            if (Storage::exists('public/images/' . $image->name))
                Storage::delete('public/images/' . $image->name);
            ImageKeyword::where('image_id', $image->id)->delete();

            $image->delete();
        }
        return ($image) ? $this->onSuccess(null) : $this->onFaile('photo not found');
    }

    public function setInformation($clientRole)
    {
        $this->clientType = 'App\\Models\\' . Str::ucfirst($clientRole);
        $this->clientEntity = auth($clientRole)->user();
        $this->clientId = $this->clientEntity->id;
        $this->client = $this->clientType::find($this->clientId);
        return 1;
    }

    public function showAddClient($clientRole, $id)
    {
        $project = Project::find($id);
//return $project;
        if (!$project)
            return $this->onSuccess($this->paginate([]));
//        return $id;
        return $this->successResponse(true, SetProjectResource::make(Project::where('id', $id)->with(['address', 'images', 'owner', 'generalContractor',
            'specialComments' => function ($q) {
//                ->whereHas('client',function ($q){
                $q->where('client_type', $this->clientType)->where('client_id', $this->clientId);
//        });
            }])->first()), ' successfully');
    }

    public function addClient($clientRole, $id, AddMemberToProjectRequest $request)
    {
//        return Str::ucfirst('generalContractor');
        if ($request->image) {
            if (is_array($request->image)) {
                foreach ($request['image'] as $keys => $images) {
                    $image = Storage::exists('/temp/images/' . $images['name']);
                    if (!$image)
                        return $this->onFaile(' photo not find ');
                }
            } else {
                return $this->onFaile('invalid photo ');
            }
        }
        $project = Project::find($id);
        if (!$project)
            return $this->onFaile('project not found');

        $this->setInformation($clientRole);

        $participant = ParticipantsProject::where('project_id', $id)->where('participants_project_type', $this->clientType)->where('participants_project_id', $this->clientId)->first();
        if ($participant)
            return $this->onFaile('You are already a member');

        $specialComment = new SpecialCommentController();
        if ($request['project_owner.id'])
            if ($request['project_owner.text'] && $request['project_owner.rate']) {
                $participantSpecialComment = SpecialComment::where('project_id', $id)->where('client_type', $this->clientType)->where('profile_type', Owner::class)->where('client_id', $this->clientId)->delete();
//                if ($participantSpecialComment)
//                return 1;
                $specialComment->store($request['project_owner'], $id, $this->clientType, $this->clientId, $request['project_owner.id'], Owner::class);
            }

        if ($request['project_general_contractor.id'] && $request['project_general_contractor.type']) {
            $participantSpecialComment = SpecialComment::where('project_id', $id)->where('client_type', $this->clientType)->where('profile_type', GeneralContractor::class)->where('client_id', $this->clientId)->delete();
            $specialComment->store($request['project_general_contractor'], $id, $this->clientType, $this->clientId, $request['project_general_contractor.id'], $this->getPathClass($request['project_general_contractor.type']));
        }
        $this->addPhotoToProject($id);

        $table = DB::table('participants_project')->insert([
            'participants_project_type' => $this->clientType,
            'participants_project_id' => $this->clientId,
            'project_id' => $id,
//            'role' => ParticipantsProject::CREATORE_ROLE,
//            'job_title_id' => $this->request['job_title_id'],
            'start_date' => $this->request['start_date'],
            'end_date' => $this->request['end_date'],
        ]);

        return $this->showAddClient($clientRole, $id);
    }

    public function showAllProjectForOthers(ShowAllProjectForOthersRequest $request)
    {

//        $clientEntity = auth($clientRole)->user();
        $clientId = $request->id;
        $clientType = $this->getPathClass($request->type);
//        return $clientId;
        $project =
            $this->pipelineWithoutPaginate(Project::query()->whereHas('participants', function ($q) use ($clientId, $clientType) {
                $q->where('participants_project_type', $clientType)->where('participants_project_id', $clientId);
            })
                ->with(['participants.industry', 'participants.participantsProject', 'images.keywords'])
                , [Condition::class]);

        return ($project) ? $this->onSuccess(new PaginationResource($this->paginate($project), ShowListProjectResource::collection($project))) : $this->onSuccess($this->paginate([]));
    }
    public function showNotRegister(ShowNotRegisterRequest $request){

      $data=  $this->getPathClass($request->type)::query()->with('specialCommentsProfileMany.client.jobTitle','specialCommentsProfileMany')->withAvg('specialCommentsProfileMany', 'rate')->where('id',$request->id)->first();
        return   ($data)? $this->onSuccess(ShowNotRegisterResource::make(($data))) :$this->onFaile('client not found');
    }
    public function listProjects(){
       $project= $this->pipeline(Project::query()->with('images'),[Name::class,ReferenceNumber::class]);
       return ($project)?$this->onSuccess(new PaginationResource($project,ListProjectOnlyShowResource::collection($project))):$this->onSuccess('error');
    }
}
