<?php

namespace App\Http\Controllers;

use App\Http\Requests\ReportRequest;
use App\Models\Report;
use App\Repositories\ReportRepository;
use App\Services\ImageService;
use App\Traits\ApiResponser;
use Illuminate\Http\Request;
use Illuminate\Support\Str;
use Symfony\Component\HttpFoundation\Response;


class ReportController extends Controller
{
    use ApiResponser;

    public Request $request;
    public $reportRepository;

    public function __construct(ReportRepository $reportRepository)
    {
        $this->reportRepository = $reportRepository;
        $this->middleware('client');
    }

    public function store(ReportRequest $reportRequest, ImageController $imageController, $clientRole)
    {


        $clientId = auth($clientRole)->user()->id;
        $clientType = 'App\\Models\\' . Str::ucfirst($clientRole);

        $imageService = new ImageService;


        if (array_key_exists('image', $reportRequest->validated())) {
            foreach ($reportRequest->validated()['image'] as $image) {
                $imageExist = $imageService->checkExist($image);
                if (!$imageExist)
                    return successResponse(false, null, 'not found this image: ' . $image);
            }
            //add report
            $repository = $this->reportRepository->storeReport($reportRequest->validated(), $clientId, $clientType);


            //add image to database and public folder
            foreach ($reportRequest->validated()['image'] as $image) {
                $imageMove = $imageService->moveTempToPublic($image);
                $imageData = $imageService->insertImage($image, Report::class, $repository->id, $clientType, $clientId);
            }
        }
        return successResponse(true,$repository, 'you create new report ');
    }

    public function showAll($clientRole)
    {
        $clientId = auth($clientRole)->user()->id;

        $clientType = 'App\\Models\\' . Str::ucfirst($clientRole);

        $repository = $this->reportRepository->showReport($clientType, $clientId);
        return ($repository)?$this->successResponse(true, $repository, 'ok'):$this->successResponseWithoutMessage(true,[]);
    }

    public function show($clientRole, $id)
    {
        $clientId = auth($clientRole)->user()->id;
        $clientType = 'App\\Models\\' . Str::ucfirst($clientRole);

        $report = $this->reportRepository->exists($id, $clientType, $clientId);

        if ($report){
            return $this->successResponse(true, $report, 'send successfully');
        }

        return $this->failedResponse('not found any report', Response::HTTP_NOT_FOUND);
    }
}
