<?php

namespace App\Http\Requests;

use Illuminate\Foundation\Http\FormRequest;

class ProjectUpdateRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules()
    {
//        return $this();

        return [
            'image'=>['nullable','array'],
            'project_name' => ['required','min:3', 'max:200'],
            'project_type' => ['required','min:3', 'string','in:commercial,residential'],
            'project_condition' => [ 'required','in:completed,inProgress,upcoming'],
            'project_priority' => [ 'required','in:special,normal'],
            'project_start' => ['required', 'date'],
            'project_end' => ['date','after:project_start'],
//            'image'=>['array'],
            'project_address.country' => ['required', 'min:2', 'max:60'],
            'project_address.city' => ['required', 'min:2', 'max:60'],
            'project_address.longitude' => ['required', 'min:2', 'max:60'],
            'project_address.latitude' => ['required', 'min:2', 'max:60'],
            'project_address.state' => ['required', 'min:2', 'max:60'],
            'project_address.zip_code' => ['required', 'numeric', 'digits_between:1,20'],
            'project_address.main_text' => ['required', 'min:2', 'max:100'],
            'project_address.secondary_text' => ['min:2', 'max:100'],


            'project_general_contractor.id_general' => ['required_with:project_general_contractor.type', 'max:255'],
            'project_general_contractor.type' => ['required_with:project_general_contractor.id_general', 'min:2', 'max:255'],

            'project_general_contractor.name' => ['required_with:project_general_contractor.email','required_with:project_general_contractor.phone_number',  'min:2', 'max:200'],
            'project_general_contractor.phone_number' => ['required_with:project_general_contractor.name','required_with:project_general_contractor.email','min:2', 'max:20'],
            'project_general_contractor.email' => ['required_with:project_general_contractor.name','required_with:project_general_contractor.phone_number', 'min:2', 'max:200', 'email'],

            'start_date' => ['required', 'date'],

            'project_style_id'=>['exists:styles,id'],


            'project_owner.first_name' => ['required_with:project_owner.last_name','required_with:project_owner.phone_number','required_with:project_owner.email','min:2', 'max:200'],
            'project_owner.last_name' => ['required_with:project_owner.first_name','required_with:project_owner.phone_number','required_with:project_owner.email','min:2', 'max:200'],
            'project_owner.phone_number' => ['required_with:project_owner.first_name','required_with:project_owner.email','required_with:project_owner.last_name','min:2', 'max:20'],
            'project_owner.email' => ['required_with:project_owner.last_name','required_with:project_owner.phone_number','required_with:project_owner.first_name', 'min:2', 'max:200', 'email'],
            'project_owner.address.main_text' => ['min:2', 'max:200'],
            'project_owner.address.secondary_text' => ['min:2', 'max:200'],
            'project_owner.address.country' => ['min:2', 'max:200'],
            'project_owner.address.state' => ['min:2', 'max:200'],
            'project_owner.address.city' => ['min:2', 'max:200'],
            'project_owner.address.zip_code' => ['numeric', 'digits_between:1,10'],
            'project_owner.address.latitude' => ['required_with:project_owner.address.longitude', 'regex:/^[-]?((((1[0-7][0-9])|([0-9]?[0-9]))\.(\d+))|180(\.0+)?)$/'],
            'project_owner.address.longitude' => ['required_with:project_company.address.latitude', 'regex:/^[-]?(([0-8]?[0-9])\.(\d+))|(90(\.0+)?)$/'],
            'project_owner.message.text' => ['min:2', 'max:500'],
            'project_owner.message.rate' => ['numeric', 'digits_between:5,10'],
            'project_owner.message.notify' => ['boolean'],

        ];
    }
}
